//
//  MSBlocks.h
//  Sketch
//
//  Created by Pieter Omvlee on 21-09-09.
//  Copyright 2009 Bohemian Coding. All rights reserved.
//

#define BLOCK void (^)()

#import "NSObject+BlockObservation.h"
  
FOUNDATION_STATIC_INLINE void CHDelay(void(^block)(void))
{
  [[NSOperationQueue mainQueue] addOperationWithBlock:block];
}

#import <Cocoa/Cocoa.h>

@interface NSImage (NSImage_Blocks)
+ (NSImage *)drawImageWithSize:(NSSize)aSize withBlock:(BLOCK)block;
+ (NSImage *)drawFlippedImageWithSize:(NSSize)aSize withBlock:(BLOCK)block;
@end

@interface NSAffineTransform (NSAffineTransform_Blocks)
- (void)transactionWithBlock:(BLOCK)block;
@end

@interface NSMenu (NSMenu_Blocks)
+ (NSMenu *)menuFromArray:(NSArray *)anArray
             blockPerItem:(void (^)(NSMenuItem *item, id object))block;
+ (NSMenu *)menuFromArray:(NSArray *)anArray
             addItemBlock:(void (^)(NSMenu *menu, id object))block;
@end

@interface NSArray (NSArray_SmallBlocks)
- (void)each:(void (^)(id object))block;
- (void)eachConcurrent:(void (^)(id object))block;
- (void)eachWithIndex:(void (^)(id object, NSUInteger index))block;
- (NSArray *)filterUsingBlock:(BOOL (^)(id object))block;
@end

@interface NSSet (NSSet_SmallBlocks)
- (void)each:(void (^)(id object))block;
@end

@interface NSArray (NSArray_Blocks)
- (NSArray *)map:(id (^)(id object))block;
- (NSArray *)mapConcurrent:(id (^)(id object))block;
- (id)reduce:(id (^)(id one, id two))block;

- (id)reduceFrom:(id)start block:(id (^)(id current, id accumulated))block;

- (NSArray *)withoutLast;

- (id)firstObject;
- (id)lastObjectOrNil;
- (id)objectAtIndexOrNil:(NSUInteger)anIndex;
+ (NSArray *)arrayWithCapacity:(NSUInteger)count fill:(id (^)(NSUInteger index))block;
@end

@interface NSObject (NSObject_NSArray)
- (void)observeKeyPath:(NSString *)keyPath block:(BLOCK)block;
- (NSArray *)asArray;
@end

@interface MSRange : NSObject
+ (void)from:(NSInteger)fromIndex to:(NSInteger)toIndex do:(void (^)(NSInteger index))block;
@end

@interface NSGraphicsContext (Blocks)
+ (void)transactionWithBlock:(BLOCK)block;
@end

@interface NSObject (NSObject_SmallBlocks)
- (void)changeKey:(NSString *)aKey inBlock:(BLOCK)block;
- (void)changeKeys:(NSArray *)keys inBlock:(BLOCK)block;
@end

@interface NSUndoManager (NSUndoManager_Blocks)
- (void)disableUndoRegistrationsDuringBlock:(BLOCK)block;
@end
